@@ -15,9 +15,9 @@ use lib 'blib/lib', 'blib/arch';
 use strict;
 use warnings;
 use Carp;
-use Algorithm::DecisionTree 3.30;
+use Algorithm::DecisionTree 3.42;
 
-our $VERSION = '3.30';
+our $VERSION = '3.42';
 
 ############################################   Constructor  ##############################################
 sub new { 
@@ -34,6 +34,7 @@ sub new {
         _csv_columns_for_features     =>  $args{csv_columns_for_features} || undef,
         _how_many_bags                =>  $args{how_many_bags} || croak("you must specify how_many_bags"),
         _bag_overlap_fraction         =>  $args{bag_overlap_fraction} || 0.20, 
+        _csv_cleanup_needed           =>  $args{csv_cleanup_needed} || 0,
         _debug1                       =>  $args{debug1} || 0,
         _number_of_training_samples   =>  undef,
         _segmented_training_data      =>  {},
@@ -59,7 +60,7 @@ sub get_training_data_for_bagging {
     while (<FILEIN>) {
         next if /^[ ]*\r?\n?$/;
         $_ =~ s/\r?\n?$//;
-        my $record = cleanup_csv($_);
+        my $record = $self->{_csv_cleanup_needed} ? cleanup_csv($_) : $_;
         if ($record_index == 0) {
             $firstline = $record;
             $record_index++;
@@ -370,6 +371,7 @@ sub check_for_illegal_params {
                             csv_columns_for_features
                             symbolic_to_numeric_cardinality_threshold
                             number_of_histogram_bins
+                            csv_cleanup_needed
                             debug1
                             debug2
                             debug3
@@ -398,17 +400,9 @@ sub fisher_yates_shuffle {
     }
 }
 
-##  Introduced in Version 3.21, I wrote this function in response to a need to
-##  create a decision tree for a very large national econometric database.  The
-##  fields in the CSV file for this database are allowed to be double quoted and such
-##  fields may contain commas inside them.  This function also replaces empty fields
-##  with the generic string 'NA' as a shorthand for "Not Available".  IMPORTANT: This
-##  function skips over the first field in each record.  It is assumed that the first
-##  field in the first record that defines the feature names is the empty string ("")
-##  and the same field in all other records is an ID number for the record.
 sub cleanup_csv {
     my $line = shift;
-    $line =~ tr/()[]{}'/       /;
+    $line =~ tr/\/:?()[]{}'/          /;
     my @double_quoted = substr($line, index($line,',')) =~ /\"[^\"]+\"/g;
     for (@double_quoted) {
         my $item = $_;
@@ -417,14 +411,14 @@ sub cleanup_csv {
         $item = join '_',  split /\s+/, $item;
         substr($line, index($line, $_), length($_)) = $item;
     }
-    my @white_spaced = $line =~ /,\s*[^,]+\s+[^,]+\s*,/g;
+    my @white_spaced = $line =~ /,(\s*[^,]+)(?=,|$)/g;
     for (@white_spaced) {
         my $item = $_;
-        $item = substr($item, 0, -1);
-        $item = join '_',  split /\s+/, $item unless $item =~ /,\s+$/;
-        substr($line, index($line, $_), length($_)) = "$item,";
+        $item =~ s/\s+/_/g;
+        $item =~ s/^\s*_|_\s*$//g;
+        substr($line, index($line, $_), length($_)) = $item;
     }
-    $line =~ s/,\s*(?=,)/,NA/g;
+    $line =~ s/,\s*(?=,|$)/,NA/g;
     return $line;
 }
 
