@@ -16,9 +16,9 @@ use strict;
 use warnings;
 use Carp;
 use List::Util qw(pairmap);
-use Algorithm::DecisionTree 3.30;
+use Algorithm::DecisionTree 3.42;
 
-our $VERSION = '3.30';
+our $VERSION = '3.42';
 
 ############################################   Constructor  ##############################################
 sub new { 
@@ -31,9 +31,10 @@ sub new {
     croak "\nYou have used a wrong name for a keyword argument --- perhaps a misspelling\n" 
                            if check_for_illegal_params(@params) == 0;
     bless {
+        _all_trees              =>  {map {$_ => Algorithm::DecisionTree->new(%dtargs)} 0..$args{how_many_trees}-1},
+        _csv_cleanup_needed                    =>  $args{csv_cleanup_needed} || 0,
         _looking_for_needles_in_haystack       =>  $args{looking_for_needles_in_haystack}, 
         _how_many_training_samples_per_tree    =>  $args{how_many_training_samples_per_tree},
-        _all_trees              =>  {map {$_ => Algorithm::DecisionTree->new(%dtargs)} 0..$args{how_many_trees}-1},
         _training_datafile                     =>  $args{training_datafile}, 
         _csv_class_column_index                =>  $args{csv_class_column_index} || undef,
         _csv_columns_for_features              =>  $args{csv_columns_for_features} || undef,
@@ -57,7 +58,7 @@ sub get_training_data_for_N_trees {
     while (<FILEIN>) {
         next if /^[ ]*\r?\n?$/;
         $_ =~ s/\r?\n?$//;
-        my $record = cleanup_csv($_);
+        my $record = $self->{_csv_cleanup_needed} ? cleanup_csv($_) : $_;
         push @{$self->{_all_record_ids}}, substr($record, 0, index($record, ','));
         $record_index++;
     }
@@ -89,7 +90,7 @@ sub get_training_data_for_N_trees_balanced {
             $i++;
             next;
         }
-        my $record = cleanup_csv($_);
+        my $record = $self->{_csv_cleanup_needed} ? cleanup_csv($_) : $_;
         my @parts = split /,/, $record;
         my $classname = $parts[$self->{_csv_class_column_index}];
         push @class_names, $classname;
@@ -104,8 +105,10 @@ sub get_training_data_for_N_trees_balanced {
     $|--;
     my @unique_class_names = sort keys %unique_class_names;
     my $num_unique_classnames = @unique_class_names;
-    die "looking_for_needles_in_haystack' option has only been tested for the case of " .
-        "two data classes.  You have $num_unique_classnames data classes. Aborting." if @unique_class_names > 2;
+    die "\n\n'looking_for_needles_in_haystack' option has only been tested for the case of " .
+        "two data classes.  You appear to have $num_unique_classnames data classes. If you know " .
+        "that you have specified only two classes, perhaps you need to use the constructor option " .
+        "'csv_cleanup_needed'. Aborting." if @unique_class_names > 2;
     print "\n\nunique class names: @unique_class_names\n" if $self->{_debug1};
     my %hist = map {$_ => 0} @unique_class_names;
     foreach my $item (@class_names) {
@@ -168,7 +171,7 @@ sub _digest_training_data_all_trees {
     while (<FILEIN>) {
         next if /^[ ]*\r?\n?$/;
         $_ =~ s/\r?\n?$//;
-        my $record = cleanup_csv($_);
+        my $record = $self->{_csv_cleanup_needed} ? cleanup_csv($_) : $_;
         if ($record_index == 0) {
             $firstline = $record;
             $record_index++;
@@ -439,6 +442,7 @@ sub check_for_illegal_params {
                             how_many_trees
                             how_many_training_samples_per_tree
                             looking_for_needles_in_haystack
+                            csv_cleanup_needed
                             debug1
                           /;
     my $found_match_flag;
@@ -455,17 +459,9 @@ sub check_for_illegal_params {
     return $found_match_flag;
 }
 
-##  Introduced in Version 3.21, I wrote this function in response to a need to
-##  create a decision tree for a very large national econometric database.  The
-##  fields in the CSV file for this database are allowed to be double quoted and such
-##  fields may contain commas inside them.  This function also replaces empty fields
-##  with the generic string 'NA' as a shorthand for "Not Available".  IMPORTANT: This
-##  function skips over the first field in each record.  It is assumed that the first
-##  field in the first record that defines the feature names is the empty string ("")
-##  and the same field in all other records is an ID number for the record.
 sub cleanup_csv {
     my $line = shift;
-    $line =~ tr/()[]{}'/       /;
+    $line =~ tr/\/:?()[]{}'/          /;
     my @double_quoted = substr($line, index($line,',')) =~ /\"[^\"]+\"/g;
     for (@double_quoted) {
         my $item = $_;
@@ -474,14 +470,14 @@ sub cleanup_csv {
         $item = join '_',  split /\s+/, $item;
         substr($line, index($line, $_), length($_)) = $item;
     }
-    my @white_spaced = $line =~ /,\s*[^,]+\s+[^,]+\s*,/g;
+    my @white_spaced = $line =~ /,(\s*[^,]+)(?=,|$)/g;
     for (@white_spaced) {
         my $item = $_;
-        $item = substr($item, 0, -1);
-        $item = join '_',  split /\s+/, $item unless $item =~ /,\s+$/;
-        substr($line, index($line, $_), length($_)) = "$item,";
+        $item =~ s/\s+/_/g;
+        $item =~ s/^\s*_|_\s*$//g;
+        substr($line, index($line, $_), length($_)) = $item;
     }
-    $line =~ s/,\s*(?=,)/,NA/g;
+    $line =~ s/,\s*(?=,|$)/,NA/g;
     return $line;
 }
 
