@@ -16,12 +16,12 @@ use strict;
 use warnings;
 use Carp;
 use File::Basename;
-use Algorithm::DecisionTree;
+use Algorithm::DecisionTree 3.42;
 use List::Util qw(reduce min max pairmap sum);
 use Math::GSL::Matrix;
 use Graphics::GnuplotIF;
 
-our $VERSION = '3.41';
+our $VERSION = '3.42';
 
 @Algorithm::RegressionTree::ISA = ('Algorithm::DecisionTree');
 
@@ -74,7 +74,7 @@ sub get_training_data_for_regression {
     while (<FILEIN>) {
         next if /^[ ]*\r?\n?$/;
         $_ =~ s/\r?\n?$//;
-        my $record = cleanup_csv($_);
+        my $record = $self->{_csv_cleanup_needed} ? cleanup_csv($_) : $_;
         if ($record_index == 0) {
             $firstline = $record;
             $record_index++;
@@ -796,7 +796,7 @@ sub bulk_predictions_for_data_in_a_csv_file {
     while (<FILEIN>) {
         next if /^[ ]*\r?\n?$/;
         $_ =~ s/\r?\n?$//;
-        my $record = cleanup_csv($_);
+        my $record = $self->{_csv_cleanup_needed} ? cleanup_csv($_) : $_;
         if ($record_index == 0) {
             @fieldnames =   grep $_, split /,/, substr($record, index($record,','));
             $record_index++;
@@ -1094,6 +1094,7 @@ sub check_for_illegal_params {
                             mse_threshold
                             need_data_normalization
                             jacobian_choice
+                            csv_cleanup_needed
                             debug1_r
                             debug2_r
                             debug3_r
@@ -1112,17 +1113,9 @@ sub check_for_illegal_params {
     return $found_match_flag;
 }
 
-##  Introduced in Version 3.21, I wrote this function in response to a need to
-##  create a decision tree for a very large national econometric database.  The
-##  fields in the CSV file for this database are allowed to be double quoted and such
-##  fields may contain commas inside them.  This function also replaces empty fields
-##  with the generic string 'NA' as a shorthand for "Not Available".  IMPORTANT: This
-##  function skips over the first field in each record.  It is assumed that the first
-##  field in the first record that defines the feature names is the empty string ("")
-##  and the same field in all other records is an ID number for the record.
 sub cleanup_csv {
     my $line = shift;
-    $line =~ tr/()[]{}'/       /;
+    $line =~ tr/\/:?()[]{}'/          /;
     my @double_quoted = substr($line, index($line,',')) =~ /\"[^\"]+\"/g;
     for (@double_quoted) {
         my $item = $_;
@@ -1131,14 +1124,14 @@ sub cleanup_csv {
         $item = join '_',  split /\s+/, $item;
         substr($line, index($line, $_), length($_)) = $item;
     }
-    my @white_spaced = $line =~ /,\s*[^,]+\s+[^,]+\s*,/g;
+    my @white_spaced = $line =~ /,(\s*[^,]+)(?=,|$)/g;
     for (@white_spaced) {
         my $item = $_;
-        $item = substr($item, 0, -1);
-        $item = join '_',  split /\s+/, $item unless $item =~ /,\s+$/;
-        substr($line, index($line, $_), length($_)) = "$item,";
+        $item =~ s/\s+/_/g;
+        $item =~ s/^\s*_|_\s*$//g;
+        substr($line, index($line, $_), length($_)) = $item;
     }
-    $line =~ s/,\s*(?=,)/,NA/g;
+    $line =~ s/,\s*(?=,|$)/,NA/g;
     return $line;
 }
 
